@@ -28,7 +28,7 @@ class ClientApp : public QApplication
 {
     Q_OBJECT
 public:
-    ClientApp(int &argc, char **argv ) : QApplication( argc, argv ) {}
+    ClientApp(int &argc, char **argv );
 
     /** Parse command-line arguments and "do it" */
     static bool doIt();
@@ -41,9 +41,10 @@ private Q_SLOTS:
     void deref();
 
 private:
-    bool kde_open( const KUrl& url, const QString& mimeType );
+    bool kde_open( const KUrl& url, const QString& mimeType, bool allowExec );
     bool doCopy( int firstArg );
     bool doMove( int firstArg );
+    bool doRemove( int firstArg );
 
     static bool m_ok;
 };
