@@ -19,30 +19,35 @@
 #ifndef _NEPOMUK_STRIGI_SYSTRAY_H_
 #define _NEPOMUK_STRIGI_SYSTRAY_H_
 
-#include <KSystemTrayIcon>
+#include <KStatusNotifierItem>
 
 class KToggleAction;
 
 namespace Nepomuk {
 
-    class IndexScheduler;
+    class StrigiService;
 
-    class SystemTray : public KSystemTrayIcon
+    class SystemTray : public KStatusNotifierItem
     {
         Q_OBJECT
 
     public:
-        SystemTray( IndexScheduler* scheduler, QWidget* parent );
+        SystemTray( StrigiService* service, QWidget* parent );
         ~SystemTray();
 
     private Q_SLOTS:
         void slotUpdateStrigiStatus();
         void slotConfigure();
+        void slotSuspend( bool suspended );
 
     private:
         KToggleAction* m_suspendResumeAction;
 
-        IndexScheduler* m_indexScheduler;
+        StrigiService* m_service;
+        bool m_suspendedManually;
+
+        // used to prevent endless status updates
+        QString m_prevStatus;
     };
 }
 
